const config = require('config')
const { last } = require('lodash')

const unsubscribeSlug = config.get('unsubscribe.url')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

module.exports = {
  async sendNotifications({
    reason,
    baseUrl,
    invitedHE,
    collection,
    isEiC = false,
    isCanceled = false,
    isAccepted = false,
    models: { User: UserModel, Fragment: FragmentModel },
  }) {
    const fragmentId = last(collection.fragments)
    const fragment = await FragmentModel.find(fragmentId)
    const fragmentHelper = new Fragment({ fragment })
    const { title } = await fragmentHelper.getFragmentData()
    const { submittingAuthor } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const titleText = `the manuscript titled "${title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.lastName}`

    const userHelper = new User({ UserModel })
    const eic = await userHelper.getEditorInChief()
    const eicName = `${eic.firstName} ${eic.lastName}`
    const subjectBaseText = `${collection.customId}: Manuscript `

    const email = new Email({
      type: 'user',
      content: {
        signatureName: eicName,
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
        ctaText: 'MANUSCRIPT DETAILS',
      },
    })

    if (isEiC) {
      sendInvitedHEEmail({
        email,
        baseUrl,
        eicName,
        isCanceled,
        titleText,
        subjectBaseText,
        handlingEditor: invitedHE,
      })
    } else {
      sendEiCEmail({
        eic,
        email,
        baseUrl,
        comments: reason ? `Reason: "${reason}"` : '',
        titleText,
        isAccepted,
        subjectBaseText,
        targetUserName: `${invitedHE.firstName} ${invitedHE.lastName}`,
      })
    }
  },
}

const sendInvitedHEEmail = ({
  email,
  eicName,
  baseUrl,
  titleText,
  isCanceled,
  handlingEditor,
  subjectBaseText,
}) => {
  email.toUser = {
    email: handlingEditor.email,
    name: `${handlingEditor.firstName} ${handlingEditor.lastName}`,
  }

  email.content.subject = isCanceled
    ? `${subjectBaseText} Assignment Revoked`
    : `${subjectBaseText} Assignment`
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
  })
  email.content.signatureName = eicName

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType: isCanceled ? 'he-revoked' : 'he-assigned',
      titleText,
    }),
  })

  email.sendEmail({ html, text })
}

const sendEiCEmail = async ({
  eic,
  email,
  baseUrl,
  comments,
  titleText,
  isAccepted,
  targetUserName,
  subjectBaseText,
}) => {
  email.content.subject = `${subjectBaseText} Assignment Response`
  const emailType = isAccepted ? 'he-accepted' : 'he-declined'

  email.toUser = {
    email: eic.email,
    name: `${eic.firstName} ${eic.lastName}`,
  }

  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: eic.id,
  })

  const { html, text } = email.getBody({
    body: getEmailCopy({
      comments,
      emailType,
      titleText,
      targetUserName,
    }),
  })

  email.sendEmail({ html, text })
}
